<?php
/**
* Implementation of input check functions
*
* Check if strings for example match email address syntax
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: sys_checkit.php,v 1.8 2006/04/12 14:12:33 rekowski Exp $
*/

/**
* Examine a string to given properties like email, numbers etc.
* This class will be used directly. You call the include functions with checkit::xyz.
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class checkit {

  /**
  * Check if string is NOT empty
  *
  * @param string $str string to check
  * @return boolean
  */
  function filled($str) {
    return (strlen(trim((string)$str)) > 0);
  }

  /**
  * Check string against the pattern (PCRE)
  * @param string $str string to check
  * @param string $pattern filter
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function check($str, $pattern, $mustContainValue = FALSE) {
    if (checkit::filled($str)) {
      $result = (preg_match((string)$pattern,(string)$str));
    } else {
      $result = (!($mustContainValue));
    }
    return (boolean)$result;
  }


  /**
  * Check string consists of letter or alfanumeric characters.
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isAlpha($str, $mustContainValue = FALSE) {
    return (checkit::check($str, "/^[a-zA-Z\. ,_-]+$/", $mustContainValue));
  }

  /**
  * Check string consists of letter or numbers.
  * @param string $str string
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isAlphaChar($str, $mustContainValue = FALSE) {
    return (checkit::check($str, "/^[\da-zA-Z\.\(\)\[\]\/ ,_-]+$/", $mustContainValue));
  }


  /**
  * Check string consists of numbers
  * @param string $str string
  * @param boolean $mustContainValue string may be empty?
  * @param integer $digitsMin minimum number of numbers
  * @param integer $digitsMax maximum number of numbers
  * @return boolean
  */
  function isNum($str, $mustContainValue = FALSE, $digitsMin = NULL, $digitsMax = NULL) {
    if (isset($digitsMin)) {
      if (isset($digitsMax)) {
        return (checkit::check($str, "/^[\+\-]?\d{".(int)$digitsMin.",".(int)$digitsMax."}\$/", $mustContainValue));
      } else {
        return (checkit::check($str, "/^[\+\-]?\d{".(int)$digitsMin."}\$/", $mustContainValue));
      }
    } else {
      return (checkit::check($str, "/^[\+\-]?\d+$/", $mustContainValue));
    }
  }

  /**
  * check string is float number (with .)
  *
  * @param string $str
  * @param boolean $mustContainValue optional, default value FALSE
  * @access public
  * @return boolean
  */
  function isFloat($str, $mustContainValue = FALSE) {
    return (checkit::check($str,"/^\d+(\.\d+)?$/", $mustContainValue));
  }

  /**
  * Check string consists of alphanumeric characters
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isAlphaNum($str, $mustContainValue = FALSE) {
    return (checkit::check($str, "/^[a-zA-Z0-9\. ,_-]+$/", $mustContainValue));
  }

  /**
  * Check string consists of alphanumeric characters with numbers
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isAlphaNumChar($str, $mustContainValue = FALSE) {
    return (checkit::check($str, "/^[a-zA-Z0-9\.\(\)\[\]\/ ,_-]+$/", $mustContainValue));
  }

  /**
  * Check string consists of measure and numbers
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isNumUnit($str, $mustContainValue = FALSE) {
    return (checkit::check($str, "/^((\d+(px|pt|%)?)|(\d+(\.\d+)?em))$/", $mustContainValue));
  }

  /**
  * Check string is safe to use in an url
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function internalSafeURL($str, $mustContainValue = FALSE) {
    return (checkit::check($str, "/^[a-zA-Z0-9\.\(\)\[\]\/ ,_-]+$/", $mustContainValue));
  }

  /**
  * Check string is no HTML
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isNoHTML($str, $mustContainValue = FALSE) {
    return (checkit::check($str,"/^[^<>]+$/",$mustContainValue));
  }

  /**
  * Check string is some text
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isSomeText($str, $mustContainValue = FALSE) {
    return (checkit::check($str, "/\S/", $mustContainValue));
  }

  /**
  * Check string is a phone number
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isPhone($str, $mustContainValue = FALSE) {
    return (checkit::check($str, "/^[0-9\+][0-9\(\)\/ -]+$/", $mustContainValue));
  }

  /**
  * Check string is filename
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isFile($str, $mustContainValue = FALSE) {
    return (checkit::check($str, "#^(([\.a-z0-9/_-]*/)*)([a-z0-9_-]+\.[a-z0-9]+)$#", $mustContainValue));
  }

  /**
  * Check string is path
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isPath($str, $mustContainValue = FALSE) {
    return (checkit::check($str, "#^([a-zA-Z]:/)?([\.a-zA-Z0-9/_-]*/)+$#", $mustContainValue));
  }

  /**
  * Check string is date in german format dd.mm.yyyy.
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isGermanDate($str, $mustContainValue = FALSE) {
    return (checkit::check($str, "#^\d{1,2}\.\d{1,2}\.\d{2,4}$#", $mustContainValue));
  }

  /**
  * Extended HTTP check
  *
  * anchors             http://www.blah.de/index.html#top
  * parameters          http://www.blah.de/index.html?foo=bar
  * virtual directories http://www.blah.de/~user/index.html
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isHTTPX($str, $mustContainValue = FALSE) {
    return (checkit::check($str, '#^https?://([a-z_]+(:[^@]+)?@)?([a-z\d-_]+\.)*([a-z\d-_]+)(\.[a-z]{2,6})?(/[%\.a-z\d_~-]*)*(\?(([:a-z\d\.\[\]%-]+)|(&?[+:a-z\d\.\[\]%-]+=[+:a-z\d\.\[\]%-]*))*)?(\#\w+)?$#i', $mustContainValue));
  }

  /**
  * Check web adress (http://* or www.*)
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isHTTP($str, $mustContainValue = FALSE) {
    return (
      (checkit::check($str, '#^http://[a-z\d_-]+\.[a-z\d_-]*\.?[a-z\d_-]+[a-z\d/\._~-]*$#i', $mustContainValue)) ||
      (checkit::check($str, '#^www\.[a-z\d_-]+\.[a-z\d9_-]+[a-z\d/\._-]*$#i', $mustContainValue))
    );
  }

  /**
  * Check http host name
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isHTTPHost($str, $mustContainValue = FALSE) {
    $specialChars = '';
    return (checkit::check($str, '#^([a-zA-Z'.$specialChars.'\d_-]+\.)*([a-zA-Z'.$specialChars.'\d-]{2,})(\.[a-z]{2,6})?#', $mustContainValue));
  }

  /**
  * Check string is email adress
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isEmail($str, $mustContainValue = FALSE) {
    return (checkit::check($str,
                  '#^[-!\#$%&\'*+\\./0-9=?A-Z^_`a-z{|}~]+@[-!\#$%&\'*+\\/0-9=?A-Z^_`a-z{|}~]+\.[-!\#$%&\'*+\\./0-9=?A-Z^_`a-z{|}~]+$#',
      $mustContainValue));
  }

  /**
  * Check string is 32 byte hexcode
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isGUID($str,$mustContainValue=FALSE) {
    return (checkit::check($str, '#^[a-fA-F\d]{32}$#', $mustContainValue));
  }

  /**
  * Check date is in ISO-format
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isISODate($str, $mustContainValue = FALSE) {
    return (checkit::check($str, '#^([12]\d{3})-(\d|(0\d)|(1[0-2]))-(([012]?\d)|(3[01]))$#', $mustContainValue));
  }

  /**
  * Check date and time is in ISO-format
  *
  * @param string $str string
  * @param boolean $mustContainValue string may be empty ?
  * @return mixed FALSE or int
  */
  function isISODateTime($str, $mustContainValue = FALSE) {
    return (checkit::check($str, '#^([12]\d{3})-(\d|(0\d)|(1[0-2]))-(([012]?\d)|(3[01])) ([01]\d|2[0-4]):([0-5]\d)$#', $mustContainValue));
  }

  /**
  * Check string is a time
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isTime($str,$mustContainValue=FALSE) {
    return (checkit::check($str, '#^((1\d)|(2[0-3])|(0?\d)):([0-5]\d)$#', $mustContainValue));
  }

  /**
  * Check string is HTML color
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isHTMLColor($str,$mustContainValue=FALSE) {
    return (checkit::check($str, '@^#[\da-fA-F]{6}$@', $mustContainValue));
  }

  /**
  * Check string is Password
  *
  * @param string $str string to check
  * @param boolean $mustContainValue string may be empty?
  * @return boolean
  */
  function isPassword($str, $mustContainValue = FALSE) {
    return (checkit::check($str, '~^\S{4,}$~', $mustContainValue));
  }

  /**
  * convert ISO date/time to unix timestamp
  *
  * @param string $str string to convert
  * @return integer
  */
  function convertISODateTimeToUnix($str) {
    if (checkit::filled($str)) {
      $date = preg_split('~[-: ]~', $str);
      return @mktime((int)$date[3], (int)$date[4], (int)$date[5], (int)$date[1], (int)$date[2], (int)$date[0]);
    } else {
      return 0;
    }
  }
}
?>
