<?php

/**
* Email
*
* @package papaya_base
* @author David Rekowski <rekowski@papaya-cms.com>
* @version $Id: sys_email.php,v 1.15 2006/04/25 10:18:59 weinert Exp $
*/

/**
* base class
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_object.php');
require_once(PAPAYA_INCLUDE_PATH.'system/papaya_strings.php');

/**
* Email
* @package papaya_base
* @author David Rekowski <rekowski@papaya-cms.com>
*/

class email extends base_object {

  var $headers;
  var $messages;
  var $templates;
  var $tagOpen = '{%';
  var $tagClose = '%}';

  /**
  * sends email and encodes headers if necessary
  * subject or body may be empty if templates are used; if set, they override template settings
  *
  * @return boolean
  */
  function send($to, $subject = NULL, $body = NULL) {
    $to = $this->getRcpt($to);
    $subject = $this->getSubject($subject);
    $body = str_replace(array("\r\n", "\r"), "\n", $this->getBody($body));
    if (FALSE !== isset($subject) && FALSE !== isset($body) && FALSE !== $to) {
      $this->setHeader('Content-Type', 'text/plain; charset=UTF-8');
      $this->setHeader('Content-Transfer-Encoding', '8bit');
      $result = @mail($this->encodeUTF8String($to), $this->encodeUTF8String($subject), $body, $this->getHeaders());
      if (!$result) {
        $this->addMsg(MSG_ERROR, 'Email could not be sent.');
      }
      return $result;
    }
    return FALSE;
  }

  /**
  * returns mailto: link with body and subject to call users email client (use in <a href="%s">link</a>)
  *
  * @uses email::getSubject()
  * @uses email::getBody()
  * @param string $to receipients email adress, may be empty
  * @param string $subject subject of message, taken from $this->templates if empty
  * @param string $to receipients email adress, may be empty
  * @return string mailto string
  */
  function getMailtoLink($to = NULL, $subject = NULL, $body = NULL) {
    $link = sprintf('mailto:%s?', rawurlencode($to));
    $subject = $this->getSubject($subject);
    $body = $this->getBody($body);
    if (FALSE === $subject) {
      $this->addMsg(MSG_WARNING, 'Subject for mailto-link not set.');
    } else {
      $link .= sprintf('subject=%s&', $this->encodeUTF8String($subject));
    }
    if (FALSE === $body) {
      $this->addMsg(MSG_WARNING, 'Body for mailto-link not set.');
    } else {
      $link .= sprintf('body=%s&', $this->encodeUTF8String($body));
    }
    if (NULL === $to) {
      $this->addMsg(MSG_WARNING, 'Receipient for mailto-link not set.');
    }
    return substr($link, 0, -1); // removes trailing ? or &
  }

  /**
  * checks receipient for linebreak chars or invalid email address format
  *
  * @param string $to receipient email address
  * @return string $to receipient email address if valid, else false
  */
  function getRcpt($to = NULL) {
    if ($to == NULL || trim($to) == '') {
      $this->addMsg(MSG_WARNING, 'Receipient is empty');
      $to = FALSE;
    }
    if (FALSE !== strpos($to, "\n") || FALSE !== strpos('$to', "\r")) {
      $this->addMsg(MSG_ERROR, 'Receipient contains linbebreak chars.');
      $to = FALSE;
    }
    if (!checkit::isEmail($to)) {
      $this->addMsg(MSG_ERROR, 'Receipient is no valid email address.');
      $to = FALSE;
    }
    return $to;
  }

  /**
  * get subject from template and checks for linebreak chars
  *
  * @param string $subject subject (empty if template is used)
  * @return string $subject subject
  */
  function getSubject($subject = NULL) {
    if (is_array($this->templates)) {
      if (isset($this->templates['subject']) && !isset($subject)) {
        $subject = $this->templates['subject'];
      }
    }
    if (FALSE !== strpos($subject, "\n") || FALSE !== strpos($subject, "\r")) {
      $this->addMsg(MSG_ERROR, "Subject contains linbebreak chars.");
      return FALSE;
    }
    return papaya_strings::ensureUTF8($subject);
  }

  /**
  * get body from template
  *
  * @param string $body body (empty if template is used)
  * @return string $body body
  */
  function getBody($body = NULL) {
    if (is_array($this->templates) && isset($this->templates['body']) && !isset($body)) {
      $body = $this->templates['body'];
    } elseif ($body == NULL) {
      $this->addMsg(MSG_ERROR, "Body not set.");
      return FALSE;
    }
    return papaya_strings::ensureUTF8($body);
  }

  /**
  * adds header name/value pair
  *
  * @param string $name name of header (i.e. what stands before ':')
  * @param string $value content of header (i.e. what stands after ':')
  */
  function setHeader($name, $value) {
    if (FALSE !== strpos($value, "\n") || FALSE !== strpos($value, "\r")) {
      $this->addMsg(MSG_ERROR, "Header contains linebreak chars.");
      return FALSE;
    }
    $this->headers[] = array($name, $value);
  }

  /**
  * returns headers formatted for use with mail() if set
  *
  * according to {@link http://de.php.net/function.mail} \r\n is used
  *
  * @return string $header formatted header
  */
  function getHeaders() {
    if (!isset($this->headers) || !is_array($this->headers) || sizeof($this->headers) == 0) {
      return '';
    }
    $header = '';
    foreach($this->headers as $entry) {
      $header .= sprintf("%s: %s\n", $entry[0], $this->encodeUTF8String($entry[1]));
    }
    return $header;
  }


  /**
  * set subject or body using template
  *
  * @uses $this->templates;
  * @param string $name must be 'body' or 'subject' in order to be used
  */
  function setTemplate($name, $template, $fillValues) {
    if ($name != 'body' && $name != 'subject') {
      $this->addMsg(MSG_WARNING, 'Template must be for body or subject');
      return FALSE;
    }
    if (is_array($fillValues) && count($fillValues) > 0) {
      foreach($fillValues as $key => $value) {
        $template = str_replace($this->tagOpen.$key.$this->tagClose, $value, $template);
      }
    }
    $this->templates[$name] = $template;
  }

  /**
  * encodes UTF8 string to RFC-2047 compatible ASCII string i.e. =?UTF-8?B?NNNNNNNNNNN?=
  *
  * @param string $str string to be encoded
  * @return string $str encoded string (if $str was not plain ascii)
  */
  function encodeUTF8String($str) {
    if (!papaya_strings::isAscii($str) && trim($str) != '') {
      $str = base64_encode($str);
      $str = sprintf('=?UTF-8?B?'.'%s'.'?=', $str);
    }
    return $str;
  }

  /**
  * reverses encodeUTF8String()
  *
  * @param string $str string to be decoded
  * @return string $str decoded string
  */
  function decodeUTF8String($str) {
    if (papaya_strings::isAscii($str) && trim($str) != '') {
      $str = substr($str, 10, -2);
      $str = base64_decode($str);
    }
    return $str;
  }
}